@@ -3,14 +3,23 @@ import shlex
 import subprocess
 import sys
 from pathlib import Path
+from typing import Dict, List
 
 import click
 import tomlkit
+from rich.status import Status
 
 from .term import term
 
 
-def get_fastqs(src):
+def get_fastqs(src: Path) -> List[Path]:
+    """determine fastq files
+    Args:
+        src: Input directory that contains fastq files.
+    Returns:
+        List of fastq files (may be gzipped).
+    """
+
     fastqs = [f for f in src.iterdir() if not f.name.startswith(".")]
     if not fastqs:
         term.print(f"Source dir: {src}, appears to be empty...")
@@ -20,36 +29,45 @@ def get_fastqs(src):
     for f in fastqs:
 
         if not any(f.name.endswith(suffix) for suffix in [".fastq", ".fastq.gz"]):
-            print(
+            term.print(
                 f"[InputError]: There is a non fastq file in the provided fastq directory: {f}",
                 err=True,
             )
-            print("Exiting.")
+            term.print("Exiting.")
             sys.exit(1)
 
     return fastqs
 
 
-def convert_to_csv(in_file, out_file):
+def fastq_to_tsv(in_file: Path, out_file: Path) -> None:
+    """convert fastq file to tsv
 
-    for i, line in enumerate(in_file):
-
-        if not line.startswith("@") or in_file[i - 1].strip() == "+":
-            continue
-        seq_id = line.strip()
-        sequence = in_file[i + 1].strip()
-        out_file.write(f"{seq_id}\t{sequence}\n")
-
-
-def fastq_to_csv(in_file, out_file):
+    Args:
+        in_file: Fastq file to convert.
+        out_file: TSV file to write to.
+    """
 
     with open(in_file) as f_in:
         with open(out_file, "w") as f_out:
-            convert_to_csv(f_in.readlines(), f_out)
+            lines = f_in.readlines()
+            for i, line in enumerate(lines):
+
+                if not line.startswith("@") or lines[i - 1].strip() == "+":
+                    continue
+                seq_id = line.strip()
+                sequence = lines[i + 1].strip()
+                f_out.write(f"{seq_id}\t{sequence}\n")
 
 
-def extract_csv_column(csv_file, column):
+def extract_csv_column(csv_file: Path, column: int) -> Path:
+    """get column from csv file
 
+    Args:
+        csv_file: File to extract column from.
+        column: Column to extract.
+    Return:
+        File containg only the extracted column.
+    """
     ext = csv_file.suffix
     tmp_out = csv_file.with_suffix(f".c{column}{ext}")
     with open(csv_file, "r") as csv_in:
@@ -61,36 +79,55 @@ def extract_csv_column(csv_file, column):
     return tmp_out
 
 
-def combine_outs(input, output):
+def get_filter_count(file_in: Path, filter_percent: float) -> int:
+    """calculate filter cutoff
 
-    files = {f.name.split(".")[0]: f for f in input.iterdir()}
-
-    term.print(f"Combing output files for {len(files)} samples.")
-
-    with output.open("w") as csv_out:
-        csv_out.write("sample\tsequence\tcount\n")
-        for sample, f in files.items():
-            with f.open("r") as csv_in:
-                for line in csv_in:
-                    csv_out.write(f"{sample}\t{line}")
-
-
-def get_filter_count(file_in, filter_percent):
-    total_reads = 0
+    Args:
+        file_in: Clustered lineage counts to filter.
+        filter_percent: Percent cutoff of total reads count.
+    Returns:
+        Minimum nominal cutoff value.
+    """
+    total_reads = 0.0
 
     with open(file_in, newline="") as csvfile:
         spamreader = csv.reader(csvfile, delimiter="\t")
         for row in spamreader:
             total_reads += float(row[1])
 
-    filter_count = int(round(total_reads * filter_percent / 100, 0))
-
-    return filter_count
+    return int(round(total_reads * filter_percent / 100, 0))
 
 
-def validate_filter_args(ctx):
+def combine_outs(input_dir: Path, output: Path) -> None:
+    """combine output tsvs into one file
+
+    Args:
+        input_dir: Directory containing csv files to combine.
+        output: TSV to save all data to.
+    """
+    files = {f.name.split(".")[0]: f for f in input_dir.iterdir()}
+
+    term.print(f"Combing output files for {len(files)} samples.")
+
+    with output.open("w") as tsv_out:
+        # TODO: make columns configurable?
+        tsv_out.write("sample\tsequence\tcount\n")
+        for sample, f in files.items():
+            with f.open("r") as tsv_in:
+                for line in tsv_in:
+                    tsv_out.write(f"{sample}\t{line}")
+
+
+def validate_filter_args(ctx: click.Context) -> Dict[str, float]:
+    """validate filter argument from config and CLI
+
+    Args:
+        ctx: Click context.
+    Returns:
+        Dictionary defining the filter type and value.
+    """
     if ctx.params["filter_count"] or ctx.params["filter_count"] == 0:
-        if ctx.get_parameter_source("filter_percent").value == 3:
+        if ctx.get_parameter_source("filter_percent").value == 3:  # type: ignore
             ctx.params["filter_percent"] = None
             del ctx.params["filter_percent"]
             return {"filter_count": ctx.params["filter_count"]}
@@ -103,7 +140,12 @@ def validate_filter_args(ctx):
         return {"filter_percent": ctx.params["filter_percent"]}
 
 
-def save_params(ctx):
+def save_params(ctx: click.Context) -> None:
+    """save parameters to config file
+
+    Args:
+        ctx: Click context.
+    """
     cmd = ctx.info_name
     params = {k: v for k, v in ctx.params.items() if v}
     save_type = params.pop("save_config")
@@ -122,22 +164,20 @@ def save_params(ctx):
         config = tomlkit.document()
 
     if save_type == "explicit":
-        params = {
-            k: v for k, v in params.items() if ctx.get_parameter_source(k).value != 3
-        }
+        params = {k: v for k, v in params.items() if ctx.get_parameter_source(k).value != 3}  # type: ignore
 
     # sanitize the path's for writing to toml
     for k in ["input", "pipeline", "output"]:
         if k in params.keys():
             params[k] = str(params[k])
 
-    config[cmd] = params
+    config[cmd] = params  # type: ignore
 
     null_hints = {"extract": ["filter_count", "fastp_args"], "merge": ["fastp_args"]}
     if cmd in null_hints.keys() and save_type == "full":
-        for param in null_hints[cmd]:
+        for param in null_hints[cmd]:  # type: ignore
             if param not in params.keys():
-                config[cmd].add(tomlkit.comment(f"{param} ="))
+                config[cmd].add(tomlkit.comment(f"{param} ="))  # type: ignore
 
     with config_file.open("w") as f:
         f.write(tomlkit.dumps(config))
@@ -146,7 +186,15 @@ def save_params(ctx):
     ctx.exit()
 
 
-def load_params(ctx, param, filename):
+def load_params(ctx: click.Context, param: str, filename: Path) -> None:
+    """load parameters from config file
+
+    Args:
+        ctx: Click context.
+        param: Invoked command and config table head.
+        filename: Config filename.
+    """
+
     if not filename or ctx.resilient_parsing:
         return
 
@@ -166,12 +214,30 @@ def load_params(ctx, param, filename):
     ctx.obj = {"config_file": filename}
 
 
-def exit_status(p, file):
+def exit_status(p: subprocess.CompletedProcess, file: Path) -> bool:
+    """check command exit status and file size
+
+    Args:
+        p: Completed subprocess
+        file: File to check for nonzero size.
+    Returns:
+        True for success, False otherwise.
+    """
     return True if p.returncode != 0 or file.stat().st_size == 0 else False
 
 
-def run_cmd(command, sample, output, verbose, status):
-    """run a subcommand"""
+def run_cmd(
+    command: str, sample: str, output: Path, verbose: bool, status: Status
+) -> None:
+    """run a subcommand
+
+    Args:
+        command: Subcommand to be run in subprocess.
+        sample: Name of sample.
+        output: Directory of immediate output.
+        verbose: If true, print subcommand output.
+        status: Status to pause if writing to stderr printing needed.
+    """
     cmd_name = command.split()[0]
 
     p = subprocess.run(
@@ -180,6 +246,7 @@ def run_cmd(command, sample, output, verbose, status):
         stderr=subprocess.STDOUT,
         universal_newlines=True,
     )
+
     # remove 'progress: ##%' output from starcode
     stdout = (
         p.stdout
@@ -202,8 +269,13 @@ def run_cmd(command, sample, output, verbose, status):
         sys.exit(1)
 
 
-def confirm_samples(samples, yes):
-    """display and confirm samples"""
+def confirm_samples(samples: List[str], yes: bool) -> None:
+    """display and confirm samples
+
+    Args:
+        samples: List of samples for confirmation.
+        yes: True if --yes flag used at runtime.
+    """
 
     term.print(f"[hl]Samples[/]: {', '.join(sorted(samples))}\n")
     if not yes and not term.confirm("Continue with these samples?"):
@@ -212,14 +284,14 @@ def confirm_samples(samples, yes):
         term.print()
 
 
-def check_output(file, message):
+def check_output(file: Path, message: str) -> bool:
     """check for output file and print message
 
     Args:
-        file (path.Pathlib): resulting file of step
-        message (str): text to display related to step
+        file: Resulting file of step.
+        message: Text to display related to step.
     Returns:
-        bool
+        True for success, False otherwise.
     """
 
     if not file.is_file():
